public class Evaluator
{
	private ExpressionTokenizer tokenizer;

	/**
         Constructs an evaluator.
         @param anExpression a string containing the expression
         to be evaluated
	 */
	public Evaluator(String anExpression)
	{
		tokenizer = new ExpressionTokenizer(anExpression);
	}
	/**
    Evaluates the expression.
    @return the value of the expression.
	 */
	public double getExpressionValue()
	{
		double value = getTermValue();
		boolean done = false;
		while (!done)
		{
			String next = tokenizer.peekToken();
			if ("+".equals(next) || "-".equals(next))
			{
				tokenizer.nextToken(); // Discard "+" or "-"
				double value2 = getTermValue();
				if ("+".equals(next)) { value = value + value2; }
				else { value = value - value2; }
			}
			else if (next == null || ")".equals(next)) {
				done = true;
			}
			else 
			{
				System.out.println("Syntax Error");
				System.exit(1);
			}
		}
		return value;
	}
	/**
    Evaluates the next term found in the expression.
    @return the value of the term
	 */
	public double getTermValue()
	{
		double value = getPowerValue();
		boolean done = false;
		while (!done)
		{
			String next = tokenizer.peekToken();
			if ("*".equals(next) || "/".equals(next))
			{
				tokenizer.nextToken();
				double value2 = getPowerValue();
				if ("*".equals(next)) { value = value * value2; }
				else { value = value / value2; }
			}
			else 
			{
				done = true;
			}
		}
		return value;
	}

//	private int myPower(int x, int y) {
//		if (y<=1) { return x; }
//		return  x* myPower(x, y - 1);
//	}
//
	/**
    Evaluates the next term found in the expression.
    @return the value of the term
	 */
	public double getPowerValue()
	{
		double value = getFactorValue();
		boolean done = false;
		while (!done)
		{
			String next = tokenizer.peekToken();
			if ("^".equals(next))
			{
				tokenizer.nextToken();
				double value2 = getFactorValue();
				if ("^".equals(next)) { value = Math.pow(value, value2); }
				else {
					System.out.println("Syntax Error");
					System.exit(1);
				}
			}
			else 
			{
				done = true;
			}
		}
		return value;
	}

	/**
    Evaluates the next factor found in the expression.
    @return the value of the factor
	 */
	public double getFactorValue()
	{
		double dblValue;
		String next = tokenizer.peekToken();
		if ("(".equals(next))
		{
			tokenizer.nextToken(); // Discard "("
			dblValue = getExpressionValue();
			tokenizer.nextToken(); // Discard ")"
		}
		else
		{
			dblValue = Double.parseDouble(tokenizer.nextToken());
		}
		return dblValue;
	}
}



